#!/usr/bin/perl

###########################################################################
# $Id: accesslog.cgi,v 1.2 2002/10/01 08:11:42 matt Exp $
###########################################################################

###########################################################################
#
#	AccessLog CGI
#	Matthew Gream <matthew.gream@pobox.com>
#	August 2002
#	
#	Very simple cgi script that extracts content from httpd access logs,
#	with caller able to specify the following arguments:
#
#		b=BEGIN_TIME
#		e=END_TIME
#		q=QUIET_MODE
#
#	BEGIN_TIME and END_TIME are strings with format YYYY/MM/DD-HH:MM:SS 
#	and the defaults are all 0's for BEGIN_TIME	and all 9's for END_TIME. 
#	QUIET_MODE is either '0' for '1' (default is '0') to allow output
#	of progress mode information about the script.
#
#	The user must specify FILENAME (below) as a full path name to the
#	access log. This cannot be specified as a cgi parameter as it would
#	allow an intruder to access arbitrary files in the system.
#
#	QUIET and FILENAME can be specified in a configuration file with 
#	name in CONFIGURE (default "accesslog.conf") using format:
#
#		filename /name/of/access.log
#		quiet 1
#
#	The script examines [...date...time...] information in the access log
#	to extract items with time 't' where 't' >= 's' && 't' < 'e'.
#	
###########################################################################

use strict;
use warnings;

$ENV{PATH} = '';
use lib qw(.);
use CGI;
use CGI::Carp qw(fatalsToBrowser);


###########################################################################
# get_time_accesslog
# get_time_input
#	convert dates to YYYYMMDDHHMMSS 
###########################################################################

my %mon2d = ( 
	jan=>'01',feb=>'02',mar=>'03',apr=>'04',
	may=>'05',jun=>'06',jul=>'07',aug=>'08',
	sep=>'09', oct=>'10',nov=>'11',dec=>'12' 
); 

sub get_time_accesslog
{
	my($i, $o) = @_;
	if ($i=~m/(\d+)\/(\w+)\/(\d+):(\d+):(\d+):(\d+).*/i)
	{
		my $d = $1;
		my $x = $mon2d{lc($2)};
		my $y = $3; if ($y < 100) { $y += 2000; }
		my $h = $4;
		my $m = $5;
		my $s = $6;
		$$o = "$y$x$d$h$m$s";
		return 1;
	}
	return 0;
}
sub get_time_input
{
	my($i, $o, $t) = @_;
	if ($i=~m/(\d+)\/(\d+)\/(\d+)(.*)/i)
	{
		my $y = $1;
		my $x = $2;
		my $d = $3;
		my $h = $t;
		my $m = $t;
		my $s = $t;
		if ($4=~m/-(\d+):(\d+):(\d+).*/i)
		{
			$h = $1;
			$m = $2;
			$s = $3;
		}
		$$o = "$y$x$d$h$m$s";
		return 1;
	}
	return 0;
}


###########################################################################
# parse_accesslog
#	extract output from filename between time_begin and time_end,
#	return total count of lines in filename
###########################################################################

sub parse_accesslog
{
	my($output, $count, $filename, $time_begin, $time_end) = @_;

	open(FILE, "<$filename") 
		|| return 0;

	$$count = 0;
	while (<FILE>)
	{
		if (/^[^\[]*\[([^\]]*)\]/)
		{
			my($c);
 			if (get_time_accesslog($1, \$c))
			{
				if ($c ge $time_begin && $c lt $time_end)
				{
					push @$output, $_;
				}
			}
		}
		$$count++;
	}
	close(FILE);

	return 1;
}


###########################################################################
# extract_accesslog
#	extract contents from accesslog between two dates 
###########################################################################

sub extract_accesslog
{
	my ($filename, $datebegin, $dateend, $quiet) = @_;

	my (@output) = ();
	my ($count) = 0;
	my ($result);

	my ($l_filename) = $filename;
	my ($l_timebegin, $l_timeend);
	get_time_input($datebegin, \$l_timebegin, "00") 
		|| die("Invalid date begin");
	get_time_input($dateend, \$l_timeend, "99") 
		|| die("Invalid date end");

	print "Reading from $l_filename for <$l_timebegin> to <$l_timeend>\n"
		unless ($quiet);

	parse_accesslog(\@output, \$count, $l_filename, $l_timebegin, $l_timeend)
		|| die("Invalid access log");

	print "Obtained ", scalar(@output), " items from $count lines\n"
		unless ($quiet);

	foreach my $o (@output) 
	{
		print $o;
	}
}


###########################################################################
# definitions
###########################################################################

my ($PROGRAM)		= "AccessLog CGI";
my ($VERSION)		= "1.0";
my ($AUTHOR)		= "Matthew Gream";


###########################################################################
# configuration
###########################################################################

my ($CONFIGURE)		= "accesslog.conf";
my ($FILENAME)		= "accesslog.txt";
my ($QUIET)			= 1;
my ($DATE_BEGIN)	= "0000/00/00-00:00:00";
my ($DATE_END)		= "9999/99/99-99:99:99";

if (open(FILE, "<$CONFIGURE")) 
{
	while(<FILE>) 
	{
		if (/^#.*/) {
			;
		} elsif (/^.*filename[ \t]*([^\#]*).*$/) {
			$FILENAME = $1;
		} elsif (/^.*quiet[ \t]*([^\#]*).*$/) {
			if ($1 eq '0') { $QUIET = 0; }
			elsif ($1 eq '1') { $QUIET = 1; }
		}
	}
	close(FILE);
}


###########################################################################
# operation
###########################################################################

my $cgi = new CGI;

if (defined $cgi->param('b'))
	{ $DATE_BEGIN = $cgi->param('b'); }
if (defined $cgi->param('e'))
	{ $DATE_END = $cgi->param('e'); }
if (defined $cgi->param('q'))
	{ $QUIET = $cgi->param('q'); }

print $cgi->header('text/plain');

extract_accesslog($FILENAME, $DATE_BEGIN, $DATE_END, $QUIET);


###########################################################################

exit 0;

